import argparse



template = r"""
{
    "release_mode":"False",
    "project_id":"SFW-00C0000",
    "firmware_version":"V1.0.0.1b2",
    "before_bin_file_path":"C:\\user_path\\telink\\ruiyun_passthrough\\TLSR825X_Tmodule\\8258_module\\8258_module.bin",
    "project_path":"C:\\user_path\\telink\\ruiyun_passthrough\\TLSR825X_Tmodule\\",
    "change_log":"1.balabal;2.baxxxx;3.sadasda"
}
"""


def add_arguments(parser:argparse.ArgumentParser) -> None:
    pass


def run(option:argparse.Namespace) -> int:
    with open("fw_standard_config.json","w",encoding="utf-8") as f:
        f.write(template)
    print("standard_fw_gen init success.enjoy it.")


import json
import os


class config_file_parser:
    def __init__(self,config_file_path):
        self._config_file_path = config_file_path
        self._json_entity = {}

    def check_json_is_right(self):
        if "release_mode" not in self._json_entity:
            print("error:release_mode field is miss.")
            return 1

        if "project_id" not in self._json_entity:
            print("error:project_id field is miss.")
            return 1

        if "firmware_version" not in self._json_entity:
            print("error:firmware_version field is miss.")
            return 1

        if "before_bin_file_path" not in self._json_entity:
            print("error:before_bin_file_path field is miss.")
            return 1

        if "project_path" not in self._json_entity:
            print("error:project_path field is miss.")
            return 1

        if "change_log" not in self._json_entity:
            print("error:change_log field is miss.")
            return 1

        return 0



    def parser(self):
        with open(self._config_file_path,"r",encoding="utf-8") as f:
            self._json_entity = json.loads(f.read(1000000))
        
        if self.check_json_is_right() == 1:
            return 1

        return 0

    
    def get_json_data(self,key):
        return self._json_entity[key]


    def reset_release_mode(self):
        self._json_entity["release_mode"] = "False"
    

    def save_json_entity_to_file(self):
        self.reset_release_mode()
        with open(self._config_file_path,"w",encoding="utf-8") as f:
            f.write(json.dumps(self._json_entity,indent = 4,ensure_ascii=False))
    





import argparse
from . import move_and_gen




def add_arguments(parser: argparse.ArgumentParser) -> None:
    parser.add_argument('config_file', nargs='?', default=None)

def run(options: argparse.Namespace) -> int:
    worker = move_and_gen.standard_gen(options.config_file)
    if worker.check() == 1:
        return 0
    worker.do_work()
    print("finish.")
    return 0





from . import config_file_parser
import os
import shutil
import hashlib
import time


"""
{
    "release_mode":"False",
    "project_id":"SFW-00C0000",
    "firmware_version":"V1.0.0.1b2",
    "before_bin_file_path":"C:\\user_path\\telink\\ruiyun_passthrough\\TLSR825X_Tmodule\\8258_module\\8258_module.bin",
    "project_path":"C:\\user_path\\telink\\ruiyun_passthrough\\TLSR825X_Tmodule\\",
    "change_log":"1.balabal;2.baxxxx;3.sadasda"
}
"""


output_file_template = """
固件名称：{}
固件MD5：{}
生成日期：{}
修改内容：{}
"""



class standard_gen:
    def __init__(self,config_file_path):
        self._json_parser = config_file_parser.config_file_parser(config_file_path + '/fw_standard_config.json')

    def check(self):
        if self._json_parser.parser() == 1:
            return 1
        return 0

    def get_file_md5(self,file_path):
        with open(file_path,"rb") as f:
            data = f.read()
            ret_md5 = hashlib.md5(data).hexdigest()
            return ret_md5

    def check_new_file_right(self,before_md5,new_file_path):
        file_tmp_md5 = self.get_file_md5(new_file_path)
        if before_md5 == file_tmp_md5:
            return 0
        return 1

    def get_local_time(self):
        return time.strftime("%Y-%m-%d %H:%M:%S",time.localtime())


    def do_work(self):
        release_flag = False
        # judge uotput path
        output_path = ""
        if self._json_parser.get_json_data("release_mode") == "True":
            output_path = "proj_release"
            release_flag = True
        else:
            output_path = "proj_debug"

        # check output path is exist
        if os.path.exists(os.path.join(self._json_parser.get_json_data("project_path"),output_path)) == False:
            os.mkdir(os.path.join(self._json_parser.get_json_data("project_path"),output_path))
        
        # get before bin file crc32
        before_file_md5 = self.get_file_md5(self._json_parser.get_json_data("before_bin_file_path"))

        # cp file and change name
        new_file_path = os.path.join(self._json_parser.get_json_data("project_path"),output_path)
        new_file_name = self._json_parser.get_json_data("project_id") + "_" + self._json_parser.get_json_data("firmware_version") + ".bin"
        shutil.copy(self._json_parser.get_json_data("before_bin_file_path"),os.path.join(new_file_path,new_file_name))

        if self.check_new_file_right(before_file_md5,os.path.join(new_file_path,new_file_name)) == 1:
            print("error:copy happend error.quit.")
            return 1

        # return if release flag is false
        if release_flag == False:
            return 0

        # generate log file context
        log_context = output_file_template.format(new_file_name,before_file_md5,self.get_local_time(),self._json_parser.get_json_data("change_log"))
        # print(log_context)

        # output log context to log file
        release_path = os.path.join(self._json_parser.get_json_data("project_path"),output_path)
        log_file_path = os.path.join(release_path,"project_fw_log.txt")
        with open(log_file_path,"a+",encoding="utf-8") as f:
            f.write(log_context)

        # set release_mode to False
        self._json_parser.save_json_entity_to_file()






        


        

        



        

        
        









import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="standard_fw_gen",
    version="0.0.4",
    author="Jack",
    author_email="JackLinZero@163.com",
    description="Standard firmware generate tool",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitee.com/JackLinXX_admin/standard_fw_gen",
    project_urls={
        "Bug Tracker": "https://gitee.com/JackLinXX_admin/standard_fw_gen/issues",
    },
    entry_points={
        "console_scripts":[
            "standard_fw_gen = standard_fw_gen.standard_fw_gen:main"
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir= {
        "" : "src",
    },
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6",
)